@@ -1,21 +1,8 @@
-// MediaInfo_Internal - All info about media files
-// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -36,6 +23,7 @@
 #include "MediaInfo/File__MultipleParsing.h"
 #include "ZenLib/Dir.h"
 #include "ZenLib/File.h"
+#include "ZenLib/FileName.h"
 #if defined(MEDIAINFO_DIRECTORY_YES)
     #include "MediaInfo/Reader/Reader_Directory.h"
 #endif
@@ -48,6 +36,10 @@
 #if defined(MEDIAINFO_LIBMMS_YES)
     #include "MediaInfo/Reader/Reader_libmms.h"
 #endif
+#if defined(MEDIAINFO_IBI_YES)
+    #include "MediaInfo/Multiple/File_Ibi.h"
+#endif
+#include "MediaInfo/Multiple/File_Dxw.h"
 #include <cmath>
 #ifdef MEDIAINFO_DEBUG_WARNING_GET
     #include <iostream>
@@ -334,6 +326,27 @@ size_t MediaInfo_Internal::Open(const String &File_Name_)
 {
     Close();
 
+    //External IBI
+    #if MEDIAINFO_IBIUSAGE
+        if (Config.Ibi_UseIbiInfoIfAvailable_Get())
+        {
+            std::string IbiFile=Config.Ibi_Get();
+            if (!IbiFile.empty())
+            {
+                Info=new File_Ibi();
+                Open_Buffer_Init(IbiFile.size(), File_Name_);
+                Open_Buffer_Continue((const int8u*)IbiFile.c_str(), IbiFile.size());
+                Open_Buffer_Finalize();
+
+                if (!Get(Stream_General, 0, __T("Format")).empty() && Get(Stream_General, 0, __T("Format"))!=__T("Ibi"))
+                    return 1;
+
+                //Nothing interesting
+                Close();
+            }
+        }
+    #endif //MEDIAINFO_IBIUSAGE
+
     CS.Enter();
     MEDIAINFO_DEBUG_CONFIG_TEXT(Debug+=__T("Open, File=");Debug+=Ztring(File_Name_).c_str();)
     Config.File_Names.clear();
@@ -451,16 +464,187 @@ void MediaInfo_Internal::Entry()
     #if defined(MEDIAINFO_FILE_YES)
         else if (File::Exists(Config.File_Names[0]))
         {
-            CS.Enter();
-            if (Reader)
-            {
-                CS.Leave();
-                return; //There is a problem
-            }
-            Reader=new Reader_File();
-            CS.Leave();
+            #if defined(MEDIAINFO_REFERENCES_YES)
+                string Dxw;
+                if (Config.File_CheckSideCarFiles_Get() && !Config.File_IsReferenced_Get())
+                {
+                    FileName Test(Config.File_Names[0]);
+                    Ztring FileExtension=Test.Extension_Get();
+                    FileExtension.MakeLowerCase();
 
-            Reader->Format_Test(this, Config.File_Names[0]);
+                    if (FileExtension!=__T("cap"))
+                    {
+                        Test.Extension_Set(__T("cap"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".cap\" />\r\n";
+                    }
+                    if (FileExtension!=__T("dfxp"))
+                    {
+                        Test.Extension_Set(__T("dfxp"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".dfxp\" />\r\n";
+                    }
+                    if (FileExtension!=__T("sami"))
+                    {
+                        Test.Extension_Set(__T("sami"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".sami\" />\r\n";
+                    }
+                    if (FileExtension!=__T("sc2"))
+                    {
+                        Test.Extension_Set(__T("sc2"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".sc2\" />\r\n";
+                    }
+                    if (FileExtension!=__T("scc"))
+                    {
+                        Test.Extension_Set(__T("scc"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".scc\" />\r\n";
+                    }
+                    if (FileExtension!=__T("smi"))
+                    {
+                        Test.Extension_Set(__T("smi"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".smi\" />\r\n";
+                    }
+                    if (FileExtension!=__T("srt"))
+                    {
+                        Test.Extension_Set(__T("srt"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".srt\" />\r\n";
+                    }
+                    if (FileExtension!=__T("stl"))
+                    {
+                        Test.Extension_Set(__T("stl"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".stl\" />\r\n";
+                    }
+                    if (FileExtension!=__T("ttml"))
+                    {
+                        Test.Extension_Set(__T("ttml"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".ttml\" />\r\n";
+                    }
+                    if (FileExtension!=__T("ssa"))
+                    {
+                        Test.Extension_Set(__T("ssa"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".ssa\" />\r\n";
+                    }
+                    if (FileExtension!=__T("ass"))
+                    {
+                        Test.Extension_Set(__T("ass"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".ass\" />\r\n";
+                    }
+                    if (FileExtension!=__T("vtt"))
+                    {
+                        Test.Extension_Set(__T("vtt"));
+                        if (File::Exists(Test))
+                            Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".vtt\" />\r\n";
+                    }
+                    if (FileExtension!=__T("xml"))
+                    {
+                        Test.Extension_Set(__T("xml"));
+                        if (File::Exists(Test))
+                        {
+                            MediaInfo_Internal MI;
+                            Ztring ParseSpeed_Save=MI.Option(__T("ParseSpeed_Get"), __T(""));
+                            Ztring Demux_Save=MI.Option(__T("Demux_Get"), __T(""));
+                            MI.Option(__T("ParseSpeed"), __T("0"));
+                            MI.Option(__T("Demux"), Ztring());
+                            size_t MiOpenResult=MI.Open(Test);
+                            MI.Option(__T("ParseSpeed"), ParseSpeed_Save); //This is a global value, need to reset it. TODO: local value
+                            MI.Option(__T("Demux"), Demux_Save); //This is a global value, need to reset it. TODO: local value
+                            if (MiOpenResult)
+                            {
+                                Ztring Format=MI.Get(Stream_General, 0, General_Format);
+                                if (Format==__T("TTML"))
+                                    Dxw+=" <clip file=\""+Test.Name_Get().To_UTF8()+".xml\" />\r\n";
+                            }
+                        }
+                    }
+
+                    Ztring Name=Test.Name_Get();
+                    Ztring BaseName=Name.SubString(Ztring(), __T("_"));
+                    if (!BaseName.empty())
+                    {
+                        ZtringList List;
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_audio.mp4"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.dfxp"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.sami"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.sc2"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.scc"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.smi"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.srt"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.stl"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_sub.vtt"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.dfxp"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.sami"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.sc2"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.scc"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.smi"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.srt"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.stl"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_forcesub.vtt"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.dfxp"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.sami"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.sc2"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.scc"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.smi"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.srt"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.stl"), Dir::Include_Files);
+                        List+=Dir::GetAllFileNames(Test.Path_Get()+PathSeparator+BaseName+__T("_*_cc.vtt"), Dir::Include_Files);
+                        for (size_t Pos=0; Pos<List.size(); Pos++)
+                            Dxw+=" <clip file=\""+List[Pos].To_UTF8()+"\" />\r\n";
+                    }
+
+                    if (!Dxw.empty())
+                    {
+                        Dxw.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n"
+                                "<indexFile xmlns=\"urn:digimetrics-xml-wrapper\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"urn:digimetrics-xml-wrapper DMSCLIP.XSD\">\r\n"
+                                " <clip source=\"main\" file=\""+FileName(Config.File_Names[0]).Name_Get().To_UTF8()+"."+FileName(Config.File_Names[0]).Extension_Get().To_UTF8()+"\" />\r\n");
+                        Dxw.append("</indexFile>\r\n");
+                        Config.File_FileNameFormat_Set(__T("Dxw"));
+                    }
+                }
+
+                if (Dxw.empty())
+                {
+                    CS.Enter();
+                    if (Reader)
+                    {
+                        CS.Leave();
+                        return; //There is a problem
+                    }
+                    Reader=new Reader_File();
+                    CS.Leave();
+
+                    Reader->Format_Test(this, Config.File_Names[0]);
+                }
+                else
+                {
+                    Open_Buffer_Init(Dxw.size(), FileName(Config.File_Names[0]).Path_Get()+PathSeparator+FileName(Config.File_Names[0]).Name_Get());
+                    Open_Buffer_Continue((const int8u*)Dxw.c_str(), Dxw.size());
+                    #if MEDIAINFO_NEXTPACKET
+                        if (Config.NextPacket_Get())
+                            return;
+                    #endif //MEDIAINFO_NEXTPACKET
+                    Open_Buffer_Finalize();
+                }
+            #else //defined(MEDIAINFO_REFERENCES_YES)
+                CS.Enter();
+                if (Reader)
+                {
+                    CS.Leave();
+                    return; //There is a problem
+                }
+                Reader=new Reader_File();
+                CS.Leave();
+
+                Reader->Format_Test(this, Config.File_Names[0]);
+            #endif //defined(MEDIAINFO_REFERENCES_YES)
 
             #if MEDIAINFO_NEXTPACKET
                 if (Config.NextPacket_Get())
@@ -493,12 +677,20 @@ size_t MediaInfo_Internal::Open (const int8u* Begin, size_t Begin_Size, const in
 size_t MediaInfo_Internal::Open_Buffer_Init (int64u File_Size_, const String &File_Name)
 {
     CriticalSectionLocker CSL(CS);
+
+    if (Config.File_Names.size()<=1) //If analyzing multiple files, theses members are adapted in File_Reader.cpp
+    {
+        if (File_Size_!=(int64u)-1)
+            Config.File_Size=Config.File_Current_Size=File_Size_;
+    }
+
     if (Info==NULL)
     {
-        if (!Config.File_ForceParser_Get().empty())
+        Ztring ForceParser = Config.File_ForceParser_Get();
+        if (!ForceParser.empty())
         {
             CS.Leave();
-            SelectFromExtension(Config.File_ForceParser_Get());
+            SelectFromExtension(ForceParser);
             CS.Enter();
         }
         if (Info==NULL)
@@ -516,6 +708,23 @@ size_t MediaInfo_Internal::Open_Buffer_Init (int64u File_Size_, const String &Fi
         Info->File_Name=File_Name;
     Info->Open_Buffer_Init(File_Size_);
 
+    if (File_Name.empty())
+    {
+        #if MEDIAINFO_EVENTS
+            {
+                struct MediaInfo_Event_General_Start_0 Event;
+                memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
+                Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_General_Start, 0);
+                Event.EventSize=sizeof(struct MediaInfo_Event_General_Start_0);
+                Event.StreamIDs_Size=0;
+                Event.Stream_Size=File_Size_;
+                Event.FileName=NULL;
+                Event.FileName_Unicode=NULL;
+                Config.Event_Send(NULL, (const int8u*)&Event, sizeof(MediaInfo_Event_General_Start_0));
+            }
+        #endif //MEDIAINFO_EVENTS
+    }
+
     return 1;
 }
 
@@ -534,6 +743,16 @@ size_t MediaInfo_Internal::Open_Buffer_Init (int64u File_Size_, int64u File_Offs
         }
     #endif //MEDIAINFO_DEBUG_BUFFER
 
+    if (Config.File_Names.size()<=1) //If analyzing multiple files, theses members are adapted in File_Reader.cpp
+    {
+        if (File_Size_!=(int64u)-1)
+        {
+            Config.File_Size=Config.File_Current_Size=File_Size_;
+            if (Info && !Info->Retrieve(Stream_General, 0, General_FileSize).empty())
+                Info->Fill(Stream_General, 0, General_FileSize, File_Size_, 10, true); //TODO: avoid multiple tests of file size field, refactor it in order to have a single place for file size info
+        }
+    }
+
     if (Info==NULL || File_Size_!=(int64u)-1)
         Open_Buffer_Init(File_Size_);
 
@@ -559,9 +778,11 @@ size_t MediaInfo_Internal::Open_Buffer_Init (int64u File_Size_, int64u File_Offs
             struct MediaInfo_Event_General_Start_0 Event;
             memset(&Event, 0xFF, sizeof(struct MediaInfo_Event_Generic));
             Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_General_Start, 0);
-            Event.EventSize=sizeof(struct MediaInfo_Event_General_Move_Done_0);
+            Event.EventSize=sizeof(struct MediaInfo_Event_General_Start_0);
             Event.StreamIDs_Size=0;
             Event.Stream_Size=File_Size_;
+            Event.FileName=NULL;
+            Event.FileName_Unicode=NULL;
             Config.Event_Send(NULL, (const int8u*)&Event, sizeof(MediaInfo_Event_General_Start_0));
         }
     #endif //MEDIAINFO_EVENTS
@@ -569,6 +790,18 @@ size_t MediaInfo_Internal::Open_Buffer_Init (int64u File_Size_, int64u File_Offs
     EXECUTE_SIZE_T(1, Debug+=__T("Open_Buffer_Init, will return 1");)
 }
 
+//---------------------------------------------------------------------------
+#if MEDIAINFO_ADVANCED2
+size_t MediaInfo_Internal::Open_Buffer_SegmentChange ()
+{
+    MEDIAINFO_DEBUG_CONFIG_TEXT(Debug+=__T("Open_Buffer_SegmentChange"))
+
+    Info->Open_Buffer_SegmentChange();
+
+    return 1;
+}
+#endif //MEDIAINFO_ADVANCED2
+
 //---------------------------------------------------------------------------
 void MediaInfo_Internal::Open_Buffer_Unsynch ()
 {
@@ -688,6 +921,9 @@ size_t MediaInfo_Internal::Open_Buffer_Finalize ()
     //Cleanup
     if (!Config.File_IsSub_Get() && !Config.File_KeepInfo_Get()) //We need info for the calling parser
     {
+        #if MEDIAINFO_TRACE
+        ParserName=Ztring().From_UTF8(Info->ParserName); //Keep it in memory in case we need it after delete of Info
+        #endif //MEDIAINFO_TRACE
         delete Info; Info=NULL;
     }
     if (Config.File_Names_Pos>=Config.File_Names.size())
@@ -713,7 +949,19 @@ std::bitset<32> MediaInfo_Internal::Open_NextPacket ()
                 Demux_EventWasSent=(Reader->Format_Test_PerParser_Continue(this)==2);
                 CS.Enter();
             }
+            else
         #endif //defined(MEDIAINFO_READER_NO)
+            {
+                #if MEDIAINFO_NEXTPACKET
+                    Config.Demux_EventWasSent=false;
+                #endif //MEDIAINFO_NEXTPACKET
+                Open_Buffer_Continue(NULL, 0);
+                #if MEDIAINFO_NEXTPACKET
+                    Demux_EventWasSent=Config.Demux_EventWasSent;
+                    if (!Demux_EventWasSent)
+                #endif //MEDIAINFO_NEXTPACKET
+                        Open_Buffer_Finalize();
+            }
     }
 
     std::bitset<32> ToReturn=Info==NULL?std::bitset<32>(0x0F):Info->Status;
@@ -842,6 +1090,10 @@ Ztring MediaInfo_Internal::Get(stream_t StreamKind, size_t StreamPos, const Stri
         return Get(Stream_General, StreamPos, __T("OverallBitRate_Maximum/String"), KindOfInfo, KindOfSearch);
     if (Parameter==__T("AFD"))
         return Get(StreamKind, StreamPos, __T("ActiveFormatDescription"), KindOfInfo, KindOfSearch);
+    if (Parameter==__T("Encoded_Application") && Info && !Info->Retrieve(StreamKind, StreamPos, "Encoded_Application/String").empty())
+        return Get(StreamKind, StreamPos, __T("Encoded_Application/String"), KindOfInfo, KindOfSearch);
+    if (Parameter==__T("Encoded_Library") && Info && !Info->Retrieve(StreamKind, StreamPos, "Encoded_Library/String").empty())
+        return Get(StreamKind, StreamPos, __T("Encoded_Library/String"), KindOfInfo, KindOfSearch);
 
     CS.Enter();
     MEDIAINFO_DEBUG_CONFIG_TEXT(Debug+=__T("Get, StreamKind=");Debug+=Ztring::ToZtring((size_t)StreamKind);Debug+=__T(", StreamKind=");Debug+=Ztring::ToZtring(StreamPos);Debug+=__T(", Parameter=");Debug+=Ztring(Parameter);)
@@ -995,7 +1247,7 @@ String MediaInfo_Internal::Option (const String &Option, const String &Value)
     else if (OptionLower.find(__T("file_seek"))==0)
     {
         #if MEDIAINFO_SEEK
-            if (Reader==NULL)
+            if (Reader==NULL && Info==NULL)
                 return __T("Error: Reader pointer is empty");
 
             size_t Method=(size_t)-1;
@@ -1053,21 +1305,25 @@ String MediaInfo_Internal::Option (const String &Option, const String &Value)
             }
 
             CS.Leave();
-            size_t Result=Reader->Format_Test_PerParser_Seek(this, Method, SeekValue, ID);
+            size_t Result;
+            if (Reader)
+                Result=Reader->Format_Test_PerParser_Seek(this, Method, SeekValue, ID);
+            else
+                Result=Open_Buffer_Seek(Method, SeekValue, ID);
             CS.Enter();
             switch (Result)
             {
                 case 1  : return __T("");
                 case 2  : return __T("Invalid value");
-                #if MEDIAINFO_IBI
+                #if MEDIAINFO_IBIUSAGE
                 case 3  : return __T("Feature not supported / IBI file not provided");
                 case 4  : return __T("Problem during IBI file parsing");
-                #endif //MEDIAINFO_IBI
+                #endif //MEDIAINFO_IBIUSAGE
                 case 5  : return __T("Invalid ID");
                 case 6  : return __T("Internal error");
-                #if !MEDIAINFO_IBI
+                #if !MEDIAINFO_IBIUSAGE
                 case (size_t)-2 : return __T("Feature not supported / IBI support disabled due to compilation options");
-                #endif //MEDIAINFO_IBI
+                #endif //MEDIAINFO_IBIUSAGE
                 case (size_t)-1 : return __T("Feature not supported");
                 default : return __T("Unknown error");
             }
@@ -1075,6 +1331,12 @@ String MediaInfo_Internal::Option (const String &Option, const String &Value)
             return __T("Seek manager is disabled due to compilation options");
         #endif //MEDIAINFO_SEEK
     }
+    #if MEDIAINFO_TRACE
+        if (OptionLower.find(__T("file_details_stringpointer")) == 0 && MediaInfoLib::Config.Inform_Get()!=__T("MAXML") && (MediaInfoLib::Config.Trace_Level_Get() || MediaInfoLib::Config.Inform_Get()==__T("Details")) && !Details.empty())
+        {
+            return Ztring::ToZtring((int64u)Details.data())+__T(':')+Ztring::ToZtring((int64u)Details.size());
+        }
+    #endif //MEDIAINFO_TRACE
     else if (OptionLower.find(__T("file_"))==0)
     {
         Ztring ToReturn2=Config.Option(Option, Value);
