@@ -1,20 +1,9 @@
-// File_Mk - Info for Matroska Video/Audio files
-// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
+
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
 // Information about Matroska files
@@ -65,14 +54,16 @@ private :
     void Ebml_DocTypeVersion();
     void Ebml_DocTypeReadVersion();
     void Segment();
-    void Segment_Attachements();
-    void Segment_Attachements_AttachedFile();
-    void Segment_Attachements_AttachedFile_FileData();
-    void Segment_Attachements_AttachedFile_FileDescription();
-    void Segment_Attachements_AttachedFile_FileName();
-    void Segment_Attachements_AttachedFile_FileMimeType();
-    void Segment_Attachements_AttachedFile_FileReferral();
-    void Segment_Attachements_AttachedFile_FileUID();
+    void Segment_Attachments();
+    void Segment_Attachments_AttachedFile();
+    void Segment_Attachments_AttachedFile_FileDescription();
+    void Segment_Attachments_AttachedFile_FileName();
+    void Segment_Attachments_AttachedFile_FileMimeType();
+    void Segment_Attachments_AttachedFile_FileData();
+    void Segment_Attachments_AttachedFile_FileUID();
+    void Segment_Attachments_AttachedFile_FileReferral();
+    void Segment_Attachments_AttachedFile_FileUsedStartTime();
+    void Segment_Attachments_AttachedFile_FileUsedEndTime();
     void Segment_Chapters();
     void Segment_Chapters_EditionEntry();
     void Segment_Chapters_EditionEntry_ChapterAtom();
@@ -108,8 +99,10 @@ private :
     void Segment_Cluster_BlockGroup_BlockAdditions_BlockMore_BlockAddID();
     void Segment_Cluster_BlockGroup_BlockAdditions_BlockMore_BlockAdditional();
     void Segment_Cluster_BlockGroup_BlockDuration();
-    void Segment_Cluster_BlockGroup_ReferenceBlock();
     void Segment_Cluster_BlockGroup_ReferencePriority();
+    void Segment_Cluster_BlockGroup_ReferenceBlock();
+    void Segment_Cluster_BlockGroup_ReferenceVirtual();
+    void Segment_Cluster_BlockGroup_CodecState();
     void Segment_Cluster_BlockGroup_Slices();
     void Segment_Cluster_BlockGroup_Slices_TimeSlice();
     void Segment_Cluster_BlockGroup_Slices_TimeSlice_Duration();
@@ -126,6 +119,8 @@ private :
     void Segment_Cues_CuePoint_CueTrackPositions();
     void Segment_Cues_CuePoint_CueTrackPositions_CueTrack();
     void Segment_Cues_CuePoint_CueTrackPositions_CueClusterPosition();
+    void Segment_Cues_CuePoint_CueTrackPositions_CueRelativePosition();
+    void Segment_Cues_CuePoint_CueTrackPositions_CueDuration();
     void Segment_Cues_CuePoint_CueTrackPositions_CueBlockNumber();
     void Segment_Info();
     void Segment_Info_ChapterTranslate();
@@ -158,12 +153,12 @@ private :
     void Segment_Tags_Tag_SimpleTag_TagName();
     void Segment_Tags_Tag_SimpleTag_TagString();
     void Segment_Tags_Tag_Targets();
-    void Segment_Tags_Tag_Targets_AttachmentUID();
-    void Segment_Tags_Tag_Targets_ChapterUID();
-    void Segment_Tags_Tag_Targets_EditionUID();
-    void Segment_Tags_Tag_Targets_TargetType();
     void Segment_Tags_Tag_Targets_TargetTypeValue();
-    void Segment_Tags_Tag_Targets_TrackUID();
+    void Segment_Tags_Tag_Targets_TargetType();
+    void Segment_Tags_Tag_Targets_TagTrackUID();
+    void Segment_Tags_Tag_Targets_TagEditionUID();
+    void Segment_Tags_Tag_Targets_TagChapterUID();
+    void Segment_Tags_Tag_Targets_TagAttachmentUID();
     void Segment_Tracks();
     void Segment_Tracks_TrackEntry();
     void Segment_Tracks_TrackEntry_AttachmentLink();
@@ -172,25 +167,29 @@ private :
     void Segment_Tracks_TrackEntry_Audio_Channels();
     void Segment_Tracks_TrackEntry_Audio_OutputSamplingFrequency();
     void Segment_Tracks_TrackEntry_Audio_SamplingFrequency();
+    void Segment_Tracks_TrackEntry_CodecSettings();
+    void Segment_Tracks_TrackEntry_CodecInfoURL();
+    void Segment_Tracks_TrackEntry_CodecDownloadURL();
     void Segment_Tracks_TrackEntry_CodecDecodeAll();
     void Segment_Tracks_TrackEntry_CodecID();
     void Segment_Tracks_TrackEntry_ContentEncodings() {};
     void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding() {};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Order() {UInteger_Info();};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Scope() {UInteger_Info();};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Type() {UInteger_Info();};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Compression();
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Compression_ContentCompAlgo();
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Compression_ContentCompSettings();
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption() {};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentEncAlgo() {UInteger_Info();};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentEncKeyID() {Skip_XX(Element_Size, "Data");};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentSignature() {Skip_XX(Element_Size, "Data");};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentSigKeyID() {Skip_XX(Element_Size, "Data");};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentSigAlgo() {UInteger_Info();};
-    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_Encryption_ContentSigHashAlgo() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncodingOrder() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncodingScope() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncodingType() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentCompression();
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentCompression_ContentCompAlgo();
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentCompression_ContentCompSettings();
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption() {};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentEncAlgo() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentEncKeyID() {Skip_XX(Element_Size, "Data");};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentSignature() {Skip_XX(Element_Size, "Data");};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentSigKeyID() {Skip_XX(Element_Size, "Data");};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentSigAlgo() {UInteger_Info();};
+    void Segment_Tracks_TrackEntry_ContentEncodings_ContentEncoding_ContentEncryption_ContentSigHashAlgo() {UInteger_Info();};
     void Segment_Tracks_TrackEntry_CodecName();
     void Segment_Tracks_TrackEntry_CodecPrivate();
+    void Segment_Tracks_TrackEntry_CodecPrivate__Parse();
     void Segment_Tracks_TrackEntry_CodecPrivate_auds();
     void Segment_Tracks_TrackEntry_CodecPrivate_auds_ExtensibleWave();
     void Segment_Tracks_TrackEntry_CodecPrivate_vids();
@@ -206,6 +205,7 @@ private :
     void Segment_Tracks_TrackEntry_Name();
     void Segment_Tracks_TrackEntry_TrackNumber();
     void Segment_Tracks_TrackEntry_TrackTimecodeScale();
+    void Segment_Tracks_TrackEntry_TrackOffset();
     void Segment_Tracks_TrackEntry_TrackType();
     void Segment_Tracks_TrackEntry_TrackUID();
     void Segment_Tracks_TrackEntry_Video();
@@ -215,7 +215,14 @@ private :
     void Segment_Tracks_TrackEntry_Video_DisplayUnit();
     void Segment_Tracks_TrackEntry_Video_DisplayWidth();
     void Segment_Tracks_TrackEntry_Video_FlagInterlaced();
+    void Segment_Tracks_TrackEntry_Video_FieldOrder();
     void Segment_Tracks_TrackEntry_Video_FrameRate();
+    void Segment_Tracks_TrackEntry_Video_Colour();
+    void Segment_Tracks_TrackEntry_Video_Colour_MatrixCoefficients();
+    void Segment_Tracks_TrackEntry_Video_Colour_BitsPerChannel();
+    void Segment_Tracks_TrackEntry_Video_Colour_Range();
+    void Segment_Tracks_TrackEntry_Video_Colour_TransferCharacteristics();
+    void Segment_Tracks_TrackEntry_Video_Colour_Primaries();
     void Segment_Tracks_TrackEntry_Video_PixelCropBottom();
     void Segment_Tracks_TrackEntry_Video_PixelCropLeft();
     void Segment_Tracks_TrackEntry_Video_PixelCropRight();
@@ -234,6 +241,7 @@ private :
     {
         std::vector<int64u>     TimeCodes;
         int64u                  TimeCode_Start;
+        int64u                  TrackUID;
         File__Analyze*          Parser;
         stream_t                StreamKind;
         size_t                  StreamPos;
@@ -252,10 +260,15 @@ private :
         std::map<std::string, Ztring> Infos;
         int64u                  TrackDefaultDuration;
         std::map<int64u, int64u> Segment_Cluster_BlockGroup_BlockDuration_Counts;
+        int64u                  PixelCropBottom;
+        int64u                  PixelCropLeft;
+        int64u                  PixelCropRight;
+        int64u                  PixelCropTop;
 
         stream()
         {
             TimeCode_Start=(int64u)-1;
+            TrackUID=(int64u)-1;
             Parser=NULL;
             StreamKind=Stream_Max;
             StreamPos=0;
@@ -272,6 +285,10 @@ private :
             ContentCompSettings_Buffer_Size=0;
             ContentCompSettings_Buffer=NULL;
             TrackDefaultDuration=0;
+            PixelCropBottom=0;
+            PixelCropLeft=0;
+            PixelCropRight=0;
+            PixelCropTop=0;
         }
 
         ~stream()
@@ -307,7 +324,7 @@ private :
     int64u   TrackType;
 
     //Temp
-    std::vector<Ztring> AttachedFiles;
+    int8u   InvalidByteMax;
     int64u  Format_Version;
     int64u  TimecodeScale;
     float64 Duration;
@@ -316,7 +333,24 @@ private :
     int64u  TrackVideoDisplayHeight;
     int32u  AvgBytesPerSec;
     int64u  Segment_Cluster_TimeCode_Value;
-    bool    Cluster_AlreadyParsed;
+    size_t  Segment_Info_Count;
+    size_t  Segment_Tracks_Count;
+    size_t  Segment_Cluster_Count;
+    typedef std::map<Ztring, Ztring> tagspertrack;
+    typedef std::map<int64u, tagspertrack> tags;
+    tags    Segment_Tags_Tag_Items;
+    int64u  Segment_Tags_Tag_Targets_TagTrackUID_Value;
+    bool    CurrentAttachmentIsCover;
+    bool    CoverIsSetFromAttachment;
+    struct crc32
+    {
+        int64u  Pos;
+        int64u  From;
+        int64u  UpTo;
+        int32u  Computed;
+        int32u  Expected;
+    };
+    std::vector<crc32> CRC32Compute;
 
     //Chapters
     struct chapterdisplay
@@ -345,10 +379,23 @@ private :
     int64u              Segment_Offset_Begin;
     int64u              Segment_Offset_End;
     std::vector<int64u> Segment_Seeks;
-    int64u              Segment_Tag_TrackUID;
+    size_t              Segment_Seeks_Pos;
     std::vector<Ztring> Segment_Tag_SimpleTag_TagNames;
     int64u Segment_Cluster_BlockGroup_BlockDuration_Value;
     int64u Segment_Cluster_BlockGroup_BlockDuration_TrackNumber;
+
+    //Hints
+    size_t*                 File_Buffer_Size_Hint_Pointer;
+
+    //Helpers
+    void JumpTo(int64u GoTo);
+    void TestMultipleInstances(size_t* Instances=NULL);
+    void CRC32_Check();
+    void CRC32_Compute(int32u &CRC32, int32u Init, const int8u* Buffer_Begin, const int8u* Buffer_End);
+    void CRC32_Compute(int32u &CRC32, const int8u* Buffer_Begin, const int8u* Buffer_End);
+#if MEDIAINFO_TRACE
+    bool CRC32_Check_In_Node(const std::string& ToSearchInInfo, const std::string& info, element_details::Element_Node *node);
+#endif // MEDIAINFO_TRACE
 };
 
 } //NameSpace
