@@ -130,6 +130,7 @@ private :
     void moov_trak_mdia_minf_hmhd();
     void moov_trak_mdia_minf_nmhd();
     void moov_trak_mdia_minf_smhd();
+    void moov_trak_mdia_minf_sthd();
     void moov_trak_mdia_minf_vmhd();
     void moov_trak_mdia_minf_stbl();
     void moov_trak_mdia_minf_stbl_cslg();
@@ -141,6 +142,8 @@ private :
     void moov_trak_mdia_minf_stbl_stps();
     void moov_trak_mdia_minf_stbl_stsc();
     void moov_trak_mdia_minf_stbl_stsd();
+    void moov_trak_mdia_minf_stbl_stsd_stpp();
+    void moov_trak_mdia_minf_stbl_stsd_stpp_btrt() {moov_trak_mdia_minf_stbl_stsd_xxxx_btrt();}
     void moov_trak_mdia_minf_stbl_stsd_text();
     void moov_trak_mdia_minf_stbl_stsd_tmcd();
     void moov_trak_mdia_minf_stbl_stsd_tmcd_name();
@@ -152,15 +155,19 @@ private :
     void moov_trak_mdia_minf_stbl_stsd_xxxxText();
     void moov_trak_mdia_minf_stbl_stsd_xxxxVideo();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_alac();
+    void moov_trak_mdia_minf_stbl_stsd_xxxx_AALP();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_ACLR();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_APRG();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_ARES();
+    void moov_trak_mdia_minf_stbl_stsd_xxxx_AORD();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_avcC();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_bitr();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_btrt();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_chan();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_clap();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_colr();
+    void moov_trak_mdia_minf_stbl_stsd_xxxx_colr_nclc(bool LittleEndian=false);
+    void moov_trak_mdia_minf_stbl_stsd_xxxx_colr_prof();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_d263();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_dac3();
     void moov_trak_mdia_minf_stbl_stsd_xxxx_damr();
@@ -210,6 +217,7 @@ private :
     void moov_trak_tapt_prof();
     void moov_trak_tapt_enof();
     void moov_trak_tkhd();
+    void moov_trak_txas();
     void moov_trak_tref();
     void moov_trak_tref_chap();
     void moov_trak_tref_clcp();
@@ -224,6 +232,8 @@ private :
     void moov_trak_tref_ssrc();
     void moov_trak_tref_sync();
     void moov_trak_tref_tmcd();
+    void moov_trak_udta();
+    void moov_trak_udta_xxxx();
     void moov_udta();
     void moov_udta_AllF();
     void moov_udta_chpl();
@@ -253,6 +263,7 @@ private :
     void moov_udta_meta_ilst_xxxx_data();
     void moov_udta_meta_ilst_xxxx_mean();
     void moov_udta_meta_ilst_xxxx_name();
+    void moov_udta_meta_uuid();
     void moov_udta_ndrm();
     void moov_udta_nsav();
     void moov_udta_rtng();
@@ -314,14 +325,14 @@ private :
     float32                                 moov_trak_tkhd_Rotation;
     std::vector<std::string>                moov_udta_meta_keys_List;
     size_t                                  moov_udta_meta_keys_ilst_Pos;
-    int32u                                  TimeScale;
+    int32u                                  moov_mvhd_TimeScale;
     int32u                                  Vendor;
     Ztring                                  Vendor_Version;
     int64u                                  FirstMdatPos;
     int64u                                  LastMdatPos; //This is the position of the byte after the last byte of mdat
     int64u                                  FirstMoovPos;
     int64u                                  moof_base_data_offset;
-    int64u                                  FrameCount_MaxPerStream;
+    int32u                                  FrameCount_MaxPerStream;
     bool                                    data_offset_present;
     int64u                                  moof_traf_base_data_offset;
     int32u                                  moof_traf_default_sample_duration;
@@ -330,12 +341,14 @@ private :
     bool                                    IsSecondPass;
     bool                                    IsParsing_mdat;
     bool                                    IsFragmented;
+    size_t                                  StreamOrder;
 
     //Data
     struct stream
     {
         Ztring                  File_Name;
         std::vector<File__Analyze*> Parsers;
+        std::map<string, Ztring> Infos;
         MediaInfo_Internal*     MI;
         struct timecode
         {
@@ -348,10 +361,13 @@ private :
         timecode* TimeCode;
         stream_t                StreamKind;
         size_t                  StreamPos;
+        int32u                  hdlr_Type;
+        int32u                  hdlr_SubType;
+        int32u                  hdlr_Manufacturer;
         struct edts_struct
         {
-            int32u  Duration;
-            int32u  Delay;
+            int64u  Duration;
+            int64u  Delay;
             int32u  Rate;
         };
         std::vector<edts_struct> edts;
@@ -394,6 +410,20 @@ private :
         bool                    IsPriorityStream;
         bool                    IsFilled;
         bool                    IsChapter;
+        bool                    IsEnabled;
+        bool                    IsExcluded;
+        bool                    HasForcedSamples;
+        bool                    AllForcedSamples;
+        std::vector<int32u>     CC;
+        std::vector<int32u>     CCFor;
+        std::vector<int32u>     FallBackTo;
+        std::vector<int32u>     FallBackFrom;
+        std::vector<int32u>     Subtitle;
+        std::vector<int32u>     SubtitleFor;
+        std::vector<int32u>     Forced;
+        std::vector<int32u>     ForcedFor;
+        std::vector<int32u>     Chapters;
+        std::vector<int32u>     ChaptersFor;
         float32                 CleanAperture_Width;
         float32                 CleanAperture_Height;
         float32                 CleanAperture_PixelAspectRatio;
@@ -416,6 +446,8 @@ private :
         #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
         #if MEDIAINFO_DEMUX
             bool            PtsDtsAreSame;
+            bool            Demux_EventWasSent;
+            int32u          CodecID;
         #endif //MEDIAINFO_DEMUX
 
         stream()
@@ -424,6 +456,9 @@ private :
             TimeCode=NULL;
             StreamKind=Stream_Max;
             StreamPos=0;
+            hdlr_Type=0x00000000;
+            hdlr_SubType=0x00000000;
+            hdlr_Manufacturer=0x00000000;
             stsz_StreamSize=0;
             stsz_Sample_Size=0;
             stsz_Sample_Multiplier=1;
@@ -437,6 +472,7 @@ private :
             stts_Duration=0;
             stts_Duration_FirstFrame=0;
             stts_Duration_LastFrame=0;
+            stts_SampleDuration = 0;
             mvex_trex_default_sample_duration=0;
             mvex_trex_default_sample_size=0;
             TimeCode_TrackID=(int32u)-1;
@@ -446,6 +482,9 @@ private :
             IsPriorityStream=false;
             IsFilled=false;
             IsChapter=false;
+            IsExcluded=false;
+            HasForcedSamples=false;
+            AllForcedSamples=false;
             CleanAperture_Width=0;
             CleanAperture_Height=0;
             CleanAperture_PixelAspectRatio=0;
@@ -457,6 +496,8 @@ private :
             #endif //MEDIAINFO_DEMUX
             #if MEDIAINFO_DEMUX
                 PtsDtsAreSame=false;
+                Demux_EventWasSent=false;
+                CodecID=0x00000000;
             #endif //MEDIAINFO_DEMUX
         }
 
@@ -478,15 +519,23 @@ private :
         bool                    ReferenceFiles_IsParsing;
     #endif //MEDIAINFO_NEXTPACKET
 
+    //Hints
+    size_t* File_Buffer_Size_Hint_Pointer;
+
     //Positions
     struct mdat_Pos_Type
     {
-        int32u StreamID;
+        int64u Offset;
         int64u Size;
+        int32u StreamID;
+        int32u Reserved1;
+        int64u Reserved2;
     };
-    typedef std::map<int64u, mdat_Pos_Type> mdat_pos;
+    typedef std::vector<mdat_Pos_Type> mdat_pos;
+    static bool mdat_pos_sort (const File_Mpeg4::mdat_Pos_Type &i,const File_Mpeg4::mdat_Pos_Type &j) { return (i.Offset<j.Offset); }
     mdat_pos mdat_Pos;
-    mdat_pos::iterator mdat_Pos_Temp;
+    mdat_Pos_Type* mdat_Pos_Temp;
+    mdat_Pos_Type* mdat_Pos_Max;
     std::vector<int32u> mdat_Pos_ToParseInPriority_StreamIDs;
     bool                mdat_Pos_NormalParsing;
 
