@@ -1,20 +1,9 @@
-// File_Mxf - Info for MXF files
-// Copyright (C) 2006-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
+
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
 // Information about Mxf files
@@ -32,8 +21,11 @@
     #include <MediaInfo/Multiple/File_Ancillary.h>
 #endif //defined(MEDIAINFO_ANCILLARY_YES)
 #include "MediaInfo/MediaInfo_Internal.h"
+#include "MediaInfo/TimeCode.h"
 #include <vector>
 #include <set>
+#include <bitset>
+using namespace std;
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -72,17 +64,28 @@ public :
 
 protected :
     //Streams management
+    void Streams_Accept();
     void Streams_Fill ();
     void Streams_Finish ();
-    void Streams_Finish_Preface (int128u PrefaceUID);
-    void Streams_Finish_ContentStorage (int128u ContentStorageUID);
-    void Streams_Finish_Package (int128u PackageUID);
-    void Streams_Finish_Track (int128u TrackUID);
+    void Streams_Finish_Preface (const int128u PrefaceUID);
+    void Streams_Finish_Preface_ForTimeCode (const int128u PrefaceUID);
+    void Streams_Finish_ContentStorage (const int128u ContentStorageUID);
+    void Streams_Finish_ContentStorage_ForTimeCode (const int128u ContentStorageUID);
+    void Streams_Finish_ContentStorage_ForAS11 (const int128u ContentStorageUID);
+    void Streams_Finish_Package (const int128u PackageUID);
+    void Streams_Finish_Package_ForTimeCode (const int128u PackageUID);
+    void Streams_Finish_Package_ForAS11 (const int128u PackageUID);
+    void Streams_Finish_Track (const int128u TrackUID);
+    void Streams_Finish_Track_ForTimeCode (const int128u TrackUID, bool IsSourcePackage);
+    void Streams_Finish_Track_ForAS11 (const int128u TrackUID);
     void Streams_Finish_Essence (int32u EssenceUID, int128u TrackUID);
-    void Streams_Finish_Descriptor (int128u DescriptorUID, int128u PackageUID);
-    void Streams_Finish_Locator (int128u DescriptorUID, int128u LocatorUID);
-    void Streams_Finish_Component (int128u ComponentUID, float64 EditRate, int32u TrackID);
-    void Streams_Finish_Identification (int128u IdentificationUID);
+    void Streams_Finish_Essence_FillID (int32u EssenceUID, int128u TrackUID);
+    void Streams_Finish_Descriptor (const int128u DescriptorUID, const int128u PackageUID);
+    void Streams_Finish_Locator (const int128u DescriptorUID, const int128u LocatorUID);
+    void Streams_Finish_Component (const int128u ComponentUID, float64 EditRate, int32u TrackID, int64u Origin);
+    void Streams_Finish_Component_ForTimeCode (const int128u ComponentUID, float64 EditRate, int32u TrackID, int64u Origin, bool IsSourcePackage);
+    void Streams_Finish_Component_ForAS11 (const int128u ComponentUID, float64 EditRate, int32u TrackID, int64u Origin);
+    void Streams_Finish_Identification (const int128u IdentificationUID);
     void Streams_Finish_CommercialNames ();
 
     //Buffer - Global
@@ -107,11 +110,34 @@ protected :
     void Data_Parse();
 
     //Elements
+    void MCAChannelID();
+    void MCALabelDictionaryID();
+    void MCATagSymbol();
+    void MCATagName();
+    void GroupOfSoundfieldGroupsLinkID();
+    void MCALinkID();
+    void SoundfieldGroupLinkID();
+    void MCAPartitionKind();
+    void MCAPartitionNumber();
+    void MCATitle();
+    void MCATitleVersion();
+    void MCATitleSubVersion();
+    void MCAEpisode();
+    void MCAAudioContentKind();
+    void MCAAudioElementKind();
+    void ResourceID();
+    void NamespaceURI();
+    void UCSEncoding();
     void Filler();
     void Filler01() {Filler();}
     void Filler02() {Filler();}
     void TerminatingFiller();
     void XmlDocumentText();
+    void SubDescriptors();
+    void LensUnitMetadata();
+    void CameraUnitMetadata();
+    void UserDefinedAcquisitionMetadata();
+    void DMFiller();
     void Sequence();
     void SourceClip();
     void TimecodeComponent();
@@ -123,21 +149,35 @@ protected :
     void Identification();
     void NetworkLocator();
     void TextLocator();
+    void StereoscopicPictureSubDescriptor();
     void MaterialPackage();
     void SourcePackage();
     void EventTrack();
     void StaticTrack();
-    void Track();
+    void TimelineTrack();
     void DMSegment();
     void GenericSoundEssenceDescriptor();
     void GenericDataEssenceDescriptor();
     void MultipleDescriptor();
+    void DMSourceClip();
     void AES3PCMDescriptor();
     void WaveAudioDescriptor();
     void MPEG2VideoDescriptor();
     void JPEG2000PictureSubDescriptor();
     void VbiPacketsDescriptor();
     void AncPacketsDescriptor();
+    void MpegAudioDescriptor();
+    void PackageMarkerObject();
+    void ApplicationPlugInObject();
+    void ApplicationReferencedObject();
+    void MCALabelSubDescriptor();
+    void TimedTextDescriptor();
+    void TimedTextResourceSubDescriptor();
+    void Unknown67SubDescriptor();
+    void Mpeg4VisualSubDescriptor();
+    void AudioChannelLabelSubDescriptor();
+    void SoundfieldGroupLabelSubDescriptor();
+    void GroupOfSoundfieldGroupsLabelSubDescriptor();
     void OpenIncompleteHeaderPartition();
     void ClosedIncompleteHeaderPartition();
     void OpenCompleteHeaderPartition();
@@ -161,6 +201,9 @@ protected :
     void SDTI_ControlMetadataSet();
     void SystemScheme1();
     void DMScheme1();
+    void AS11_AAF_Core();
+    void AS11_AAF_Segmentation();
+    void AS11_AAF_UKDPP();
     void Omneon_010201010100();
     void Omneon_010201020100();
 
@@ -195,6 +238,7 @@ protected :
     void CDCIEssenceDescriptor_ReversedByteOrder();             //330B
     void ContentStorage_Packages();                             //1901
     void ContentStorage_EssenceContainerData();                 //1902
+    void DMSegment_Duration();                                  //0202 (copy of StructuralComponent_Duration) //TODO: merge with StructuralComponent_Duration
     void DMSegment_DMFramework();                               //6101
     void DMSegment_TrackIDs();                                  //6102
     void EssenceContainerData_LinkedPackageUID();               //2701
@@ -230,7 +274,7 @@ protected :
     void GenericPictureEssenceDescriptor_VideoLineMap();        //320D
     void GenericPictureEssenceDescriptor_AspectRatio();         //320E
     void GenericPictureEssenceDescriptor_AlphaTransparency();   //320F
-    void GenericPictureEssenceDescriptor_Gamma();               //3210
+    void GenericPictureEssenceDescriptor_TransferCharacteristic(); //3210
     void GenericPictureEssenceDescriptor_ImageAlignmentOffset();//3211
     void GenericPictureEssenceDescriptor_FieldDominance();      //3212
     void GenericPictureEssenceDescriptor_ImageStartOffset();    //3213
@@ -239,6 +283,8 @@ protected :
     void GenericPictureEssenceDescriptor_StoredF2Offset();      //3216
     void GenericPictureEssenceDescriptor_DisplayF2Offset();     //3217
     void GenericPictureEssenceDescriptor_ActiveFormatDescriptor();//3218
+    void GenericPictureEssenceDescriptor_ColorPrimaries();      //3219
+    void GenericPictureEssenceDescriptor_CodingEquations();     //321A
     void GenericSoundEssenceDescriptor_QuantizationBits();      //3D01
     void GenericSoundEssenceDescriptor_Locked();                //3D02
     void GenericSoundEssenceDescriptor_AudioSamplingRate();     //3D03
@@ -283,10 +329,15 @@ protected :
     void JPEG2000PictureSubDescriptor_YTOsiz();                 //8009
     void JPEG2000PictureSubDescriptor_Csiz();                   //800A
     void JPEG2000PictureSubDescriptor_PictureComponentSizing(); //800B
+    void JPEG2000PictureSubDescriptor_CodingStyleDefault();     //
+    void JPEG2000PictureSubDescriptor_QuantizationDefault();    //
+    void MpegAudioDescriptor_BitRate();                         //
     void MultipleDescriptor_SubDescriptorUIDs();                //3F01
-    void DMScheme1_PrimaryExtendedSpokenLanguage();             //
-    void DMScheme1_SecondaryExtendedSpokenLanguage();           //
-    void DMScheme1_OriginalExtendedSpokenLanguage();            //
+    void PrimaryExtendedSpokenLanguage();                       //
+    void SecondaryExtendedSpokenLanguage();                     //
+    void OriginalExtendedSpokenLanguage();                      //
+    void SecondaryOriginalExtendedSpokenLanguage();             //
+    void RFC5646AudioLanguageCode();                            //
     void MPEG2VideoDescriptor_SingleSequence();                 //
     void MPEG2VideoDescriptor_ConstantBFrames();                //
     void MPEG2VideoDescriptor_CodedContentType();               //
@@ -297,6 +348,16 @@ protected :
     void MPEG2VideoDescriptor_BPictureCount();                  //
     void MPEG2VideoDescriptor_ProfileAndLevel();                //
     void MPEG2VideoDescriptor_BitRate();                        //
+    void Mpeg4VisualDescriptor_SingleSequence() {MPEG2VideoDescriptor_SingleSequence();}
+    void Mpeg4VisualDescriptor_ConstantBFrames() {MPEG2VideoDescriptor_ConstantBFrames();}
+    void Mpeg4VisualDescriptor_CodedContentType() {MPEG2VideoDescriptor_CodedContentType();}
+    void Mpeg4VisualDescriptor_LowDelay() {MPEG2VideoDescriptor_LowDelay();}
+    void Mpeg4VisualDescriptor_ClosedGOP() {MPEG2VideoDescriptor_ClosedGOP();}
+    void Mpeg4VisualDescriptor_IdenticalGOP() {MPEG2VideoDescriptor_IdenticalGOP();}
+    void Mpeg4VisualDescriptor_MaxGOP() {MPEG2VideoDescriptor_MaxGOP();}
+    void Mpeg4VisualDescriptor_BPictureCount() {MPEG2VideoDescriptor_BPictureCount();}
+    void Mpeg4VisualDescriptor_ProfileAndLevel();               //
+    void Mpeg4VisualDescriptor_BitRate() {MPEG2VideoDescriptor_BitRate();}
     void NetworkLocator_URLString();                            //4001
     void Preface_LastModifiedDate();                            //3B02
     void Preface_ContentStorage();                              //3B03
@@ -322,7 +383,16 @@ protected :
     void SourcePackage_Descriptor();                            //4701
     void StructuralComponent_DataDefinition();                  //0201
     void StructuralComponent_Duration();                        //0202
+    void SystemScheme1_FrameCount();                            //0101
     void SystemScheme1_TimeCodeArray();                         //0102
+    void SystemScheme1_ClipIDArray();                           //0103
+    void SystemScheme1_ExtendedClipIDArray();                   //0104
+    void SystemScheme1_VideoIndexArray();                       //0105
+    void SystemScheme1_KLVMetadataSequence();                   //0106
+    void SystemScheme1_SampleRate();                            //3001
+    void SystemScheme1_EssenceTrackNumber();                    //4804
+    void SystemScheme1_EssenceTrackNumberBatch();               //6801
+    void SystemScheme1_ContentPackageIndexArray();              //6803
     void TextLocator_LocatorName();                             //4101
     void TimecodeComponent_StartTimecode();                     //1501
     void TimecodeComponent_RoundedTimecodeBase();               //1502
@@ -342,6 +412,107 @@ protected :
     void WaveAudioDescriptor_PeakEnvelopeTimestamp();           //3D30
     void WaveAudioDescriptor_PeakEnvelopeData();                //3D31
     void WaveAudioDescriptor_ChannelAssignment();               //3D31
+    void LensUnitMetadata_IrisFNumber();                        //8000
+    void LensUnitMetadata_FocusPositionFromImagePlane();        //8001
+    void LensUnitMetadata_FocusPositionFromFrontLensVertex();   //8002
+    void LensUnitMetadata_MacroSetting();                       //8003
+    void LensUnitMetadata_LensZoom35mmStillCameraEquivalent();  //8004
+    void LensUnitMetadata_LensZoomActualFocalLength();          //8005
+    void LensUnitMetadata_OpticalExtenderMagnification();       //8006
+    void LensUnitMetadata_LensAttributes();                     //8007
+    void LensUnitMetadata_IrisTNumber();                        //8008
+    void LensUnitMetadata_IrisRingPosition();                   //8009
+    void LensUnitMetadata_FocusRingPosition();                  //800A
+    void LensUnitMetadata_ZoomRingPosition();                   //800B
+    void CameraUnitMetadata_CaptureGammaEquation();             //3210
+    void CameraUnitMetadata_AutoExposureMode();                 //8100
+    void CameraUnitMetadata_AutoFocusSensingAreaSetting();      //8101
+    void CameraUnitMetadata_ColorCorrectionFilterWheelSetting();//8102
+    void CameraUnitMetadata_NeutralDensityFilterWheelSetting(); //8103
+    void CameraUnitMetadata_ImageSensorDimensionEffectiveWidth();//8104
+    void CameraUnitMetadata_ImageSensorDimensionEffectiveHeight();//8105
+    void CameraUnitMetadata_CaptureFrameRate();                 //8106
+    void CameraUnitMetadata_ImageSensorReadoutMode();           //8107
+    void CameraUnitMetadata_ShutterSpeed_Angle();               //8108
+    void CameraUnitMetadata_ShutterSpeed_Time();                //8109
+    void CameraUnitMetadata_CameraMasterGainAdjustment();       //810A
+    void CameraUnitMetadata_ISOSensitivity();                   //810B
+    void CameraUnitMetadata_ElectricalExtenderMagnification();  //810C
+    void CameraUnitMetadata_AutoWhiteBalanceMode();             //810D
+    void CameraUnitMetadata_WhiteBalance();                     //800E
+    void CameraUnitMetadata_CameraMasterBlackLevel();           //810F
+    void CameraUnitMetadata_CameraKneePoint();                  //8110
+    void CameraUnitMetadata_CameraKneeSlope();                  //8111
+    void CameraUnitMetadata_CameraLuminanceDynamicRange();      //8112
+    void CameraUnitMetadata_CameraSettingFileURI();             //8113
+    void CameraUnitMetadata_CameraAttributes();                 //8114
+    void CameraUnitMetadata_ExposureIndexofPhotoMeter();        //8115
+    void CameraUnitMetadata_GammaForCDL();                      //8116
+    void CameraUnitMetadata_ASC_CDL_V12();                      //8117
+    void CameraUnitMetadata_ColorMatrix();                      //8118
+    void UserDefinedAcquisitionMetadata_UdamSetIdentifier();    //E000
+    void UserDefinedAcquisitionMetadata_Sony_8007();
+    void UserDefinedAcquisitionMetadata_Sony_E101();
+    void UserDefinedAcquisitionMetadata_Sony_E102();
+    void UserDefinedAcquisitionMetadata_Sony_E103();
+    void UserDefinedAcquisitionMetadata_Sony_E104();
+    void UserDefinedAcquisitionMetadata_Sony_E105();
+    void UserDefinedAcquisitionMetadata_Sony_E106();
+    void UserDefinedAcquisitionMetadata_Sony_E107();
+    void UserDefinedAcquisitionMetadata_Sony_E109();
+    void UserDefinedAcquisitionMetadata_Sony_E10B();
+    void UserDefinedAcquisitionMetadata_Sony_E201();
+    void UserDefinedAcquisitionMetadata_Sony_E202();
+    void UserDefinedAcquisitionMetadata_Sony_E203();
+    void AS11_Core_SeriesTitle();
+    void AS11_Core_ProgrammeTitle();
+    void AS11_Core_EpisodeTitleNumber();
+    void AS11_Core_ShimName();
+    void AS11_Core_AudioTrackLayout();
+    void AS11_Core_PrimaryAudioLanguage();
+    void AS11_Core_ClosedCaptionsPresent();
+    void AS11_Core_ClosedCaptionsType();
+    void AS11_Core_ClosedCaptionsLanguage();
+    void AS11_Core_ShimVersion();
+    void AS11_Segment_PartNumber();
+    void AS11_Segment_PartTotal();
+    void AS11_UKDPP_ProductionNumber();
+    void AS11_UKDPP_Synopsis();
+    void AS11_UKDPP_Originator();
+    void AS11_UKDPP_CopyrightYear();
+    void AS11_UKDPP_OtherIdentifier();
+    void AS11_UKDPP_OtherIdentifierType();
+    void AS11_UKDPP_Genre();
+    void AS11_UKDPP_Distributor();
+    void AS11_UKDPP_PictureRatio();
+    void AS11_UKDPP_3D();
+    void AS11_UKDPP_3DType();
+    void AS11_UKDPP_ProductPlacement();
+    void AS11_UKDPP_FpaPass();
+    void AS11_UKDPP_FpaManufacturer();
+    void AS11_UKDPP_FpaVersion();
+    void AS11_UKDPP_VideoComments();
+    void AS11_UKDPP_SecondaryAudioLanguage();
+    void AS11_UKDPP_TertiaryAudioLanguage();
+    void AS11_UKDPP_AudioLoudnessStandard();
+    void AS11_UKDPP_AudioComments();
+    void AS11_UKDPP_LineUpStart();
+    void AS11_UKDPP_IdentClockStart();
+    void AS11_UKDPP_TotalNumberOfParts();
+    void AS11_UKDPP_TotalProgrammeDuration();
+    void AS11_UKDPP_AudioDescriptionPresent();
+    void AS11_UKDPP_AudioDescriptionType();
+    void AS11_UKDPP_OpenCaptionsPresent();
+    void AS11_UKDPP_OpenCaptionsType();
+    void AS11_UKDPP_OpenCaptionsLanguage();
+    void AS11_UKDPP_SigningPresent();
+    void AS11_UKDPP_SignLanguage();
+    void AS11_UKDPP_CompletionDate();
+    void AS11_UKDPP_TextlessElementsExist();
+    void AS11_UKDPP_ProgrammeHasText();
+    void AS11_UKDPP_ProgrammeTextLanguage();
+    void AS11_UKDPP_ContactEmail();
+    void AS11_UKDPP_ContactTelephoneNumber();
     void Omneon_010201010100_8001();                            //8001
     void Omneon_010201010100_8003();                            //8003
     void Omneon_010201020100_8002();                            //8002
@@ -375,6 +546,29 @@ protected :
     #define Info_UL(_INFO, _NAME, _PARAM) int128u _INFO;
     #endif //MEDIAINFO_TRACE
 
+    //TimeCode
+    struct mxftimecode
+    {
+        int16u  RoundedTimecodeBase;
+        int64u  StartTimecode;
+        bool    DropFrame;
+
+        mxftimecode()
+            : RoundedTimecodeBase(0)
+            , StartTimecode((int64u)-1)
+            , DropFrame(false)
+        {
+        }
+
+        mxftimecode(int16u RoundedTimecodeBase_, int64u StartTimecode_, bool DropFrame_)
+            : RoundedTimecodeBase(RoundedTimecodeBase_)
+            , StartTimecode(StartTimecode_)
+            , DropFrame(DropFrame_)
+        {
+        }
+    };
+
+    // Temp
     struct randomindexmetadata
     {
         int64u ByteOffset;
@@ -389,6 +583,7 @@ protected :
     int128u InstanceUID;
     int64u Buffer_Begin;
     int64u Buffer_End;
+    bool   Buffer_End_Unlimited;
     int64u Buffer_Header_Size;
     int16u Code2;
     int16u Length2;
@@ -398,6 +593,7 @@ protected :
     bool   IsParsingEnd;
     bool   IsCheckingRandomAccessTable;
     bool   IsCheckingFooterPartitionAddress;
+    bool   IsSearchingFooterPartitionAddress;
     bool   FooterPartitionAddress_Jumped;
     bool   PartitionPack_Parsed;
     size_t IdIsAlwaysSame_Offset;
@@ -431,6 +627,8 @@ protected :
         Ztring ProductName;
         Ztring ProductVersion;
         Ztring VersionString;
+        Ztring ToolkitVersion;
+        Ztring Platform;
         std::map<std::string, Ztring> Infos;
     };
     typedef std::map<int128u, identification> identifications; //Key is InstanceUID of identification
@@ -468,7 +666,9 @@ protected :
         int32u TrackID;
         Ztring TrackName;
         int32u TrackNumber;
+        float64 EditRate_Real; //Before demux adaptation
         float64 EditRate;
+        int64u  Origin;
         bool   Stream_Finish_Done;
 
         track()
@@ -476,7 +676,9 @@ protected :
             Sequence=0;
             TrackID=(int32u)-1;
             TrackNumber=(int32u)-1;
+            EditRate_Real=(float64)0;
             EditRate=(float64)0;
+            Origin=0;
             Stream_Finish_Done=false;
         }
     };
@@ -499,6 +701,8 @@ protected :
         bool   Track_Number_IsMappedToTrack; //if !Track_Number_IsAvailable, is true when it was euristicly mapped
         bool   IsFilled;
         bool   IsChannelGrouping;
+        int64u      Field_Count_InThisBlock_1;
+        int64u      Field_Count_InThisBlock_2;
         int64u      Frame_Count_NotParsedIncluded;
         frame_info  FrameInfo;
 
@@ -514,6 +718,8 @@ protected :
             Track_Number_IsMappedToTrack=false;
             IsFilled=false;
             IsChannelGrouping=false;
+            Field_Count_InThisBlock_1=0;
+            Field_Count_InThisBlock_2=0;
             Frame_Count_NotParsedIncluded=(int64u)-1;
             FrameInfo.DTS=(int64u)-1;
         }
@@ -526,6 +732,7 @@ protected :
     };
     typedef std::map<int32u, essence> essences; //Key is TrackNumber
     essences Essences;
+    bitset<Stream_Max+1> StreamPos_StartAtZero; //information about the base of StreamPos (0 or 1, 1 is found in 1 file) TODO: per Essence code (last 4 bytes of the Essence header 0xTTXXTTXX)
 
     //Descriptor
     struct descriptor
@@ -551,6 +758,7 @@ protected :
         int32u SubSampling_Horizontal;
         int32u SubSampling_Vertical;
         int32u ChannelCount;
+        int128u ChannelAssignment;
         std::map<std::string, Ztring> Infos;
         int16u BlockAlign;
         int32u QuantizationBits;
@@ -561,6 +769,7 @@ protected :
         enum type
         {
             Type_Unknown,
+            type_Mutiple,
             Type_CDCI,
             Type_RGBA,
             Type_MPEG2Video,
@@ -568,11 +777,35 @@ protected :
             Type_AES3PCM,
             Type_JPEG2000Picture,
             Type_AncPackets,
+            Type_MCALabelSubDescriptor,
+            Type_AudioChannelLabelSubDescriptor,
+            Type_SoundfieldGroupLabelSubDescriptor,
+            Type_GroupOfSoundfieldGroupsLabelSubDescriptor,
         };
         type Type;
         bool HasBFrames;
+        bool HasMPEG2VideoDescriptor;
+        bool IsAes3Descriptor;
         int32u ByteRate;
 
+
+        //MCALabelSubDescriptor specific (including SoundfieldGroupLabelSubDescriptor...)
+        int128u     MCALabelDictionaryID;
+        int128u     MCALinkID;
+        Ztring      MCATagSymbol;
+        Ztring      MCATagName;
+        Ztring      MCAPartitionKind;
+        Ztring      MCAPartitionNumber;
+        Ztring      MCATitle;
+        Ztring      MCATitleVersion;
+        Ztring      MCATitleSubVersion;
+        Ztring      MCAEpisode;
+        Ztring      MCAAudioContentKind;
+        Ztring      MCAAudioElementKind;
+
+        //AudioChannelLabelSubDescriptor specific
+        int128u     SoundfieldGroupLinkID;
+
         descriptor()
         {
             StreamKind=Stream_Max;
@@ -595,15 +828,29 @@ protected :
             SubSampling_Horizontal=(int32u)-1;
             SubSampling_Vertical=(int32u)-1;
             ChannelCount=(int32u)-1;
+            ChannelAssignment.hi=(int64u)-1;
+            ChannelAssignment.lo=(int64u)-1;
             BlockAlign=(int16u)-1;
-            QuantizationBits=(int8u)-1;
+            QuantizationBits=(int32u)-1;
             Duration=(int64u)-1;
             ActiveFormat=(int8u)-1;
             FieldTopness=(int8u)-1; //Field x is upper field
             FieldDominance=1; //Default is field 1 temporaly first
             Type=Type_Unknown;
             HasBFrames=false;
+            HasMPEG2VideoDescriptor=false;
+            IsAes3Descriptor=false;
             ByteRate=(int32u)-1;
+
+            //MCALabelSubDescriptor specific (including SoundfieldGroupLabelSubDescriptor...)
+            MCALabelDictionaryID.hi=(int64u)-1;
+            MCALabelDictionaryID.lo=(int64u)-1;
+            MCALinkID.hi=(int64u)-1;
+            MCALinkID.lo=(int64u)-1;
+
+            //AudioChannelLabelSubDescriptor specific
+            SoundfieldGroupLinkID.hi=(int64u)-1;
+            SoundfieldGroupLinkID.lo=(int64u)-1;
         }
     };
     typedef std::map<int128u, descriptor> descriptors; //Key is InstanceUID of Descriptor
@@ -644,21 +891,12 @@ protected :
         int256u SourcePackageID; //Sequence from SourcePackage only
         int32u  SourceTrackID;
         std::vector<int128u> StructuralComponents; //Sequence from MaterialPackage only
-
-        //Time code component
-        int16u  TimeCode_RoundedTimecodeBase;
-        int64u  TimeCode_StartTimecode;
-        bool    TimeCode_DropFrame;
+        mxftimecode MxfTimeCode;
 
         component()
         {
             Duration=(int64u)-1;
             SourceTrackID=(int32u)-1;
-
-            //Time code component
-            TimeCode_RoundedTimecodeBase=(int16u)-1;
-            TimeCode_StartTimecode=(int64u)-1;
-            TimeCode_DropFrame=false;
         }
 
         void Update (struct component &New)
@@ -671,6 +909,13 @@ protected :
                 SourceTrackID=New.SourceTrackID;
             if (!New.StructuralComponents.empty())
                 StructuralComponents=New.StructuralComponents;
+            if (New.MxfTimeCode.StartTimecode!=(int64u)-1)
+                MxfTimeCode.StartTimecode=New.MxfTimeCode.StartTimecode;
+            if (New.MxfTimeCode.RoundedTimecodeBase)
+            {
+                MxfTimeCode.RoundedTimecodeBase=New.MxfTimeCode.RoundedTimecodeBase;
+                MxfTimeCode.DropFrame=New.MxfTimeCode.DropFrame;
+            }
         }
     };
     typedef std::map<int128u, component> components; //Key is InstanceUID of the component
@@ -681,9 +926,15 @@ protected :
     {
         int128u     Framework;
         std::vector<int32u> TrackIDs;
+        int64u Duration;
+        bool    IsAs11SegmentFiller;
 
         dmsegment()
         {
+            Framework.lo=(int64u)-1;
+            Framework.hi=(int64u)-1;
+            Duration=(int64u)-1;
+            IsAs11SegmentFiller=false;
         }
 
         ~dmsegment()
@@ -709,6 +960,108 @@ protected :
     typedef std::map<int128u, dmscheme1> dmscheme1s; //Key is InstanceUID of the DMScheme1
     dmscheme1s DMScheme1s;
 
+    //Descriptive Metadata - AS11
+    struct as11
+    {
+        enum as11_type
+        {
+            Type_Unknown,
+            Type_Core,
+            Type_Segmentation,
+            Type_UKDPP,
+        };
+        as11_type Type;
+        Ztring SeriesTitle;
+        Ztring ProgrammeTitle;
+        Ztring EpisodeTitleNumber;
+        Ztring ShimName;
+        int8u  AudioTrackLayout;
+        Ztring PrimaryAudioLanguage;
+        int8u  ClosedCaptionsPresent;
+        int8u  ClosedCaptionsType;
+        Ztring ClosedCaptionsLanguage;
+        int8u  ShimVersion_Major;
+        int8u  ShimVersion_Minor;
+        int16u PartNumber;
+        int16u PartTotal;
+        Ztring ProductionNumber;
+        Ztring Synopsis;
+        Ztring Originator;
+        int16u CopyrightYear;
+        Ztring OtherIdentifier;
+        Ztring OtherIdentifierType;
+        Ztring Genre;
+        Ztring Distributor;
+        int32u PictureRatio_N;
+        int32u PictureRatio_D;
+        int8u  ThreeD;
+        int8u  ThreeDType;
+        int8u  ProductPlacement;
+        int8u  FpaPass;
+        Ztring FpaManufacturer;
+        Ztring FpaVersion;
+        Ztring VideoComments;
+        Ztring SecondaryAudioLanguage;
+        Ztring TertiaryAudioLanguage;
+        int8u  AudioLoudnessStandard;
+        Ztring AudioComments;
+        int64u LineUpStart;
+        int64u IdentClockStart;
+        int16u TotalNumberOfParts;
+        int64u TotalProgrammeDuration;
+        int8u  AudioDescriptionPresent;
+        int8u  AudioDescriptionType;
+        int8u  OpenCaptionsPresent;
+        int8u  OpenCaptionsType;
+        Ztring OpenCaptionsLanguage;
+        int8u  SigningPresent;
+        int8u  SignLanguage;
+        int64u CompletionDate;
+        int8u  TextlessElementsExist;
+        int8u  ProgrammeHasText;
+        Ztring ProgrammeTextLanguage;
+        Ztring ContactEmail;
+        Ztring ContactTelephoneNumber;
+
+        as11()
+        {
+            Type=Type_Unknown;
+            AudioTrackLayout=(int8u)-1;
+            ClosedCaptionsPresent=(int8u)-1;
+            ClosedCaptionsType=(int8u)-1;
+            ShimVersion_Major=(int8u)-1;
+            ShimVersion_Minor=(int8u)-1;
+            PartNumber=(int16u)-1;
+            PartTotal=(int16u)-1;
+            CopyrightYear=(int16u)-1;
+            PictureRatio_N=(int32u)-1;
+            PictureRatio_D=(int32u)-1;
+            ThreeD=(int8u)-1;
+            ThreeDType=(int8u)-1;
+            ProductPlacement=(int8u)-1;
+            AudioLoudnessStandard=(int8u)-1;
+            LineUpStart=(int64u)-1;
+            IdentClockStart=(int64u)-1;
+            TotalNumberOfParts=(int16u)-1;
+            TotalProgrammeDuration=(int64u)-1;
+            AudioDescriptionPresent=(int8u)-1;
+            AudioDescriptionType=(int8u)-1;
+            OpenCaptionsPresent=(int8u)-1;
+            OpenCaptionsType=(int8u)-1;
+            SigningPresent=(int8u)-1;
+            SignLanguage=(int8u)-1;
+            CompletionDate=(int64u)-1;
+            TextlessElementsExist=(int8u)-1;
+            ProgrammeHasText=(int8u)-1;
+        }
+
+        ~as11()
+        {
+        }
+    };
+    typedef std::map<int128u, as11> as11s; //Key is InstanceUID of the DMScheme1
+    as11s AS11s;
+
     //Parsers
     void           ChooseParser__FromEssence(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Aaf(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
@@ -722,6 +1075,8 @@ protected :
     void           ChooseParser__Aaf_GC_Compound(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Avid(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Avid_Picture(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser__Sony(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser__Sony_Picture(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__FromEssenceContainer(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_Avc(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
@@ -731,6 +1086,7 @@ protected :
     void           ChooseParser_Raw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_RV24(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_Vc3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_TimedText(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_Aac(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_Ac3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser_Alaw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
@@ -743,21 +1099,27 @@ protected :
 
     //Helpers
     void Subsampling_Compute(descriptors::iterator Descriptor);
-    void Locators_CleanUp();
-    void Locators_Test();
+    #if defined(MEDIAINFO_REFERENCES_YES)
+        void Locators_CleanUp();
+        void Locators_Test();
+    #else //defined(MEDIAINFO_REFERENCES_YES)
+        inline void Locators_CleanUp() {}
+        inline void Locators_Test() {}
+    #endif //defined(MEDIAINFO_REFERENCES_YES)
+    void NextRandomIndexMetadata();
     void TryToFinish();
 
     //Temp
     int128u EssenceContainer_FromPartitionMetadata;
     int64u PartitionMetadata_PreviousPartition;
     int64u PartitionMetadata_FooterPartition;
-    int64u TimeCode_StartTimecode;
-    int16u TimeCode_RoundedTimecodeBase;
-    bool   TimeCode_DropFrame;
+    int64u RandomIndexMetadatas_MaxOffset;
+    mxftimecode MxfTimeCodeForDelay;
+    mxftimecode MxfTimeCodeMaterial;
     float64 DTS_Delay; //In seconds
-    bool   StreamPos_StartAtOne; //information about the base of StreamPos (0 or 1, 1 is found in 1 file)
-    string SDTI_TimeCode_StartTimecode;
-    int64u SDTI_TimeCode_StartTimecode_ms;
+    TimeCode            SDTI_TimeCode_StartTimecode;
+    size_t              SDTI_TimeCode_RepetitionCount;
+    TimeCode            SDTI_TimeCode_Previous;
     int64u SDTI_SizePerFrame;
     bool   SDTI_IsPresent; //Used to test if SDTI packet is used for Index StreamOffset calculation
     bool   SDTI_IsInIndexStreamOffset; //Used to test if SDTI packet is used for Index StreamOffset calculation
@@ -765,8 +1127,26 @@ protected :
     int64u SystemScheme1_TimeCodeArray_StartTimecode_ms;
     int64u SystemScheme1_FrameRateFromDescriptor;
     bool   Essences_FirstEssence_Parsed;
+    bool   MayHaveCaptionsInStream;
+    bool   StereoscopicPictureSubDescriptor_IsPresent;
+    bool   UserDefinedAcquisitionMetadata_UdamSetIdentifier_IsSony;
+    int32u Essences_UsedForFrameCount;
     int32u IndexTable_NSL;
     int32u IndexTable_NPE;
+    struct systemscheme
+    {
+        bool IsTimeCode;
+
+        systemscheme()
+        {
+            IsTimeCode=false;
+        }
+    };
+    typedef std::map<int16u, systemscheme> systemschemes;
+    systemschemes SystemSchemes;
+    #if MEDIAINFO_ADVANCED
+        int64u Footer_Position;
+    #endif //MEDIAINFO_ADVANCED
     #if defined(MEDIAINFO_ANCILLARY_YES)
         File_Ancillary* Ancillary;
         bool            Ancillary_IsBinded;
@@ -807,8 +1187,62 @@ protected :
     bool                            Partitions_IsCalculatingSdtiByteCount;
     bool                            Partitions_IsFooter;
 
-    #if MEDIAINFO_DEMUX || MEDIAINFO_SEEK
+    //Config
+    bool                            TimeCodeFromMaterialPackage;
+
+    //CameraUnitMetadata
+    struct acquisitionmetadata
+    {
+        string Value;
+        size_t FrameCount;
+
+        acquisitionmetadata(const string &Value_)
+            : Value(Value_)
+            , FrameCount(1)
+        {}
+    };
+    typedef std::vector<acquisitionmetadata> acquisitionmetadatalist;
+    vector<acquisitionmetadatalist*> AcquisitionMetadataLists;
+    void AcquisitionMetadata_Add(size_t Id, const string& Value)
+    {
+        if (!AcquisitionMetadataLists[Id])
+        {
+            AcquisitionMetadataLists[Id]=new acquisitionmetadatalist;
+            AcquisitionMetadataLists[Id]->push_back(acquisitionmetadata(Value));
+            return;
+        }
+        if ((*AcquisitionMetadataLists[Id])[AcquisitionMetadataLists[Id]->size()-1].Value == Value)
+        {
+            (*AcquisitionMetadataLists[Id])[AcquisitionMetadataLists[Id]->size()-1].FrameCount++;
+            return;
+        }
+        AcquisitionMetadataLists[Id]->push_back(acquisitionmetadata(Value));
+    }
+    vector<acquisitionmetadatalist*> AcquisitionMetadata_Sony_E201_Lists;
+    void AcquisitionMetadata_Sony_E201_Add(size_t Id, const string& Value)
+    {
+        if (!AcquisitionMetadata_Sony_E201_Lists[Id])
+        {
+            AcquisitionMetadata_Sony_E201_Lists[Id]=new acquisitionmetadatalist;
+            AcquisitionMetadata_Sony_E201_Lists[Id]->push_back(acquisitionmetadata(Value));
+            return;
+        }
+        if ((*AcquisitionMetadata_Sony_E201_Lists[Id])[AcquisitionMetadata_Sony_E201_Lists[Id]->size()-1].Value == Value)
+        {
+            (*AcquisitionMetadata_Sony_E201_Lists[Id])[AcquisitionMetadata_Sony_E201_Lists[Id]->size()-1].FrameCount++;
+            return;
+        }
+        AcquisitionMetadata_Sony_E201_Lists[Id]->push_back(acquisitionmetadata(Value));
+    }
+    int8u AcquisitionMetadata_Sony_CalibrationType;
+
+    //Demux
+    #if MEDIAINFO_DEMUX
         bool Demux_HeaderParsed;
+        essences::iterator Demux_CurrentEssence;
+    #endif //MEDIAINFO_DEMUX
+
+    #if MEDIAINFO_DEMUX || MEDIAINFO_SEEK
         size_t CountOfLocatorsToParse;
         float64 Demux_Rate;
 
@@ -853,6 +1287,10 @@ protected :
         int64u  OverallBitrate_IsCbrForSure;
         bool    Duration_Detected;
         bool    DetectDuration();
+        int64u  DemuxedSampleCount_Total;
+        int64u  DemuxedSampleCount_Current;
+        int64u  DemuxedSampleCount_AddedToFirstFrame;
+        int64u  DemuxedElementSize_AddedToFirstFrame;
     #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
 };
 
